//-----------------------------------------------------------------------------
// Title      : prj_ub001_mk3_gps
//-----------------------------------------------------------------------------
// Funktion   : GPS-Modul
// Schaltung  : myAVR MK3
//-----------------------------------------------------------------------------
// Prozessor  : ATMega2560
// Takt       : 16 MHz
// Sprache    : C
// Date       : 20.03.2010
// Version    : 1.2
// Autor      : UB
//-----------------------------------------------------------------------------
#define F_CPU 16000000
#include <avr\io.h>
#include <ub_mk3_2560\MK3_2560_LCD.h>
#include <ub_mk3_2560\MK3_2560_KEY.h>
#include <ub_mk3_2560\MK3_2560_V24.h>
#include <string.h>

char buf1[RX_PUFFER_LEN+1];
char gpgga_data[16][12]; // 16 Datenwerte
char temp[12];

void checkGPGGA(void);
void printGPGGA(void);
void getGPGGAdata(int nr);

//-----------------------------------------------------------------------------
// Hinweis : Quick-Jumper Port-C+A, Port-K muss gesetzt sein
// - GPS-Modul muss an Com0 (E0,E1) angeschlossen sein
// - PC muss an Com1 (D2,D3=myAVR) angeschlossen sein
// - Mode des myAVR-USB muss auf "Data-Mode" stehen
// - Start/Stop mit Button-1
// - Danach werden die GPS-Daten am Display angezeigt
// - RS232 enable mit Button-2
// - Danach werden alle Daten auch zum PC gesendet
//-----------------------------------------------------------------------------
int main (void) {
	bool start=false;
	bool send=false;
	int v1=0;

	MK3_INIT_LCD();					// init des LCD
	MK3_INIT_KEY();					// init der Tasten
	MK3_INIT_V24(COM1,B38400);		// init der V24 am mySmartUSB MK3
	MK3_INIT_V24(COM0,B9600);			// init der V24 an Port-E (GPS-Modul)

	MK3_LCD_CLEAR();				// LCD loeschen
	MK3_LCD_LIGHT(true);				// beleuchtung einschalten

	MK3_LCD_STRING_AT(0,0,"GPS-Projekt [V1.2]");

	MK3_LCD_STRING_AT(0,16,"COM-0 (GPS) 9600B,8N1");
	MK3_LCD_STRING_AT(0,24,"COM-1 (PC) 38400B,8N1");

	MK3_LCD_STRING_AT(0,40,"Button-1 = Start");
	MK3_LCD_STRING_AT(0,48,"Button-2 = RS232");
	while (true) {
		// warte auf Taste
		if(MK3_KEY_SET(BUTTON1)) {
			MK3_LCD_CLEAR();
			MK3_LCD_STRING_AT(0,0,"GPS aktiv (GPGGA)");
			start=true;							    
		}
		if((MK3_KEY_SET(BUTTON2)) && (start==true)) {
			if(send==false) {
			  send=true;
			  MK3_LCD_STRING_AT(90,56,"RS232");
			}
			else {
			  send=false;
			  MK3_LCD_STRING_AT(90,56,"     ");
			}							    
		}
		

		// warte auf Daten vom GPS-Modul
		if(MK3_V24_IN(COM0,buf1)) {
			if(start==true) {
				if(send==true) {
					MK3_V24_OUT(COM1,buf1,RX_END_CRLF);
				}
				
				if(strncmp(buf1,"$GPGGA",6)==0) {
					v1++;
					if(v1>1) v1=0;
					if(v1==0) MK3_LCD_STRING_AT(120,0,"-");
					if(v1==1) MK3_LCD_STRING_AT(120,0,"|");
					checkGPGGA();
					printGPGGA();
				}
			}
			
		}
		
	}
	return 0;
}
//-----------------------------------------------------------------------------

void checkGPGGA(void) 
{
	int n;
	char c;
	int nr=0,pos=0;

	for(n=0;n<RX_PUFFER_LEN+1;n++) 
	{
		c=buf1[n];
		if(c==0) break;
		if((c==13) || (c==10)) break;  
		if(c==',') 
		{
			// wenn Trennungszeichen
			gpgga_data[nr][pos]=0; // endekennung
			nr++;
			pos=0;
		}
		else 
		{
			// wenn normales zeichen
			gpgga_data[nr][pos]=c; // zeichen speichern
			pos++;
		}
	}
}

void printGPGGA(void) 
{

	int n;
	char c;

	getGPGGAdata(0); // ID lesen

	if(strncmp(temp,"$GPGGA",6)==0) 
	{
		//----------------------------------
		getGPGGAdata(3); // NordSued holen
		MK3_LCD_STRING_AT(0,16,temp);
		getGPGGAdata(2); // Wert holen
		MK3_LCD_STRING_AT(10,16,temp);
		//----------------------------------
		getGPGGAdata(5); // OstWest holen
		MK3_LCD_STRING_AT(0,24,temp);
		getGPGGAdata(4); // Wert holen
		MK3_LCD_STRING_AT(10,24,temp);
		//----------------------------------
		getGPGGAdata(6); // Indikator holen
		MK3_LCD_STRING_AT(0,32,"OK=");
		MK3_LCD_PRINT_STRING(temp);
		//----------------------------------
		getGPGGAdata(7); // anz Satelliten
		MK3_LCD_PRINT_STRING("  Anzahl=");
		MK3_LCD_PRINT_STRING(temp);
		//----------------------------------
		getGPGGAdata(8); // Abweichung holen
		MK3_LCD_STRING_AT(0,40,"Abweichung=");
		MK3_LCD_PRINT_STRING(temp);
		MK3_LCD_PRINT_STRING(" [m]");
		//----------------------------------
		getGPGGAdata(9); // hoehe holen
		MK3_LCD_STRING_AT(0,48,"Hoehe=");
		MK3_LCD_PRINT_STRING(temp);
		MK3_LCD_PRINT_STRING(" [m]");
	}
}

void getGPGGAdata(int nr) 
{
	int n;
	char c;

	for(n=0;n<13;n++) 
	{
		c=gpgga_data[nr][n];
		temp[n]=c;
		if(c==0) break; 
	}
	
}

